﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;

namespace ConsoleInterface
{
    public enum ProgrammingLanguages
    {
        Fortran, Algol, Cobol, Simula, Pascal,
        Ada, C, CPlusPlus, Eiffel, Java, CSharp
    }
    
    /// <summary>
    /// Класс - наследник интерфейса IEnumerable,
    /// допускающий перечислимость объектов.
    /// Перечислимость сводится к перечислимости персон,
    /// заданных полем container
    /// </summary>
    class Persons:IEnumerable
    {
        protected int size;
        protected Person[] container;
        Random rnd = new Random(); 
        /// <summary>
        /// Конструктор по умолчанию.
        /// Создает массив из 10 персон
        /// </summary>
        public Persons()
        {
            size = 10;
            container = new Person[size];
            FillContainer();
        }
        /// <summary>
        /// Конструктор. Создает массив заданной размерности
        /// Создает его элементы, используя рандомизацию.
        /// </summary>
        /// <param name="size">размерность массива</param>
        public Persons(int size)
        {
            this.size = size;
            container = new Person[size];
            FillContainer();
        }
        /// <summary>
        /// Конструктор, которому передается массив персон
        /// </summary>
        /// <param name="container"> массив Person</param>
        public Persons(Person[] container)
        {
            this.container = container;
            size = container.Length;
        }
        /// <summary>
        /// Заполнение массива person 
        /// </summary>
        void FillContainer()
        {
            for(int i = 0; i <size; i++)
            {
                int num = rnd.Next(3*size);
                int age = rnd.Next(27, 46);
                container[i] = new Person("агент_" + num, age);
            }
        }
        
        /// <summary>
        /// Реализация метода интерфейса IEnumerable
        /// Сводится к вызову соответствующего метода 
        /// для поля container - массива,
        /// у которого этот метод реализован в библиотеке FCL 
        /// </summary>
        /// <returns>перечислитель – интерфейсный объект</returns>
        public IEnumerator GetEnumerator()
        {
            return container.GetEnumerator();
        }

        /// <summary>
        /// Перечисление элементов массива container
        /// в обратном порядке
        /// </summary>
        /// <returns> интерфейсный объект</returns>
        public IEnumerable ReverseIterator()
        {
            for (int i = container.Length - 1; i >= 0; i--)
                yield return container[i];
        }
        
        /// <summary>
        /// Итератор. Создает объект перечисления
        /// и строит контейнер, 
        /// помещая в него элементы перечисления
        /// </summary>
        /// <returns>интерфейсный объект</returns>
        public IEnumerable LanguagesIterator()
        {
            ProgrammingLanguages lang;
            int i = 0;
            while (true)
            {
                lang = (ProgrammingLanguages)i;
                yield return lang;
                if (lang.ToString() == "CSharp") yield break;                
                i++;
            }
        }

    }
}
