﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ConsoleInterface
{
    /// <summary>
    /// Аналог класса TwoInterfaces
    /// реализует методы интерфейса IStrings
    /// как закрытые - private
    /// </summary>
    class PrivateInterface : IStrings
    {
        //поля класса
        string text;
        static string[] codeTable =
        {
          "абвгдеёжзийклмнопрстуфхцчшщыьъэюя ,.!?;:",
          "ъьыщшчцхфуэюя ,.!?;:тсрпонмлкйабвгдеёжзи"
        };
        //Конструкторы
        public PrivateInterface()
        {
            text = "Простой текст!";
        }
        public PrivateInterface(string txt)
        {
            text = txt;
        }
        public string Text
        {
            get { return text; }
        }
        //Реализация интерфейсов
        /// <summary>
        /// Удаление пробелов  в поле text, 
        /// преобразование к нижнему регистру 
        /// </summary>
        /// <returns>преобразованная строка</returns>
        string IStrings.Convert()
        {
            string s = text;
            string res = "";
            foreach (char sym in s)            
                if (sym != ' ') res += sym.ToString();
            res = res.ToLower();
            return res;
        }
        /// <summary>
        /// Шифрование поля text
        /// </summary>
        /// <param name="code">таблица кодировки символов</param>
        /// <returns>зашифрованный текст</returns>
        string IStrings.Cipher(string[] code)
        {
            string s = text;
            string res = "";
            foreach (char sym in s)
            {
                int k = code[0].IndexOf(sym);
                if (k >= 0) res += code[1][k];
                else res += sym.ToString();
            }
            return res;
        }
        /// <summary>
        /// Проверка поля text,является ли он палиндромом
        ///  после преобразования Convert
        /// </summary>
        /// <returns> true, если палиндром</returns>
        public bool IsPalindrom()
        {
            string txt = ((IStrings)this).Convert();
            for(int i=0, j = txt.Length-1; i<j; i++, j--)
                if(txt[i] != txt[j]) return false;
            return true;
        }
        /// <summary>
        /// Шифрование, заданное таблицей кодировки 
        /// </summary>
        /// <returns>шифрованный текст</returns>
        public string Coding()
        {
            
            return ((IStrings)this).Cipher(codeTable);
        }
        //Обертка методов интерфейса с переименованием
        public string ClassConvert()
        {
            return ((IStrings)this).Convert();
        }
        public string ClassCipher(string[] code)
        {
            return ((IStrings)this).Cipher(code);
        } 
    }
}
