﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ConsoleInterface
{
    /// <summary>
    /// Наследует интерфейс IStrings,
    /// реализуя его методы как общедоступные (public)
    /// </summary>
    class SimpleText : IStrings
    {
        //поля класса
        string text;
        static string[] codeTable =
        {
          "абвгдеёжзийклмнопрстуфхцчшщыьъэюя ,.!?;:",
          "ъьыщшчцхфуэюя ,.!?;:тсрпонмлкйабвгдеёжзи"
        };
        //Конструкторы
        public SimpleText()
        {
            text = "Простой текст!";
        }
        public SimpleText(string txt)
        {
            text = txt;
        }
        public string Text
        {
            get { return text; }
        }
        //Реализация интерфейсов
        /// <summary>
        /// Удаление пробелов в поле text, 
        /// преобразование к нижнему регистру 
        /// </summary>
        /// <returns>преобразованная строка</returns>
        public string Convert()
        {            
            string res = "";
            foreach (char sym in text)            
                if (sym != ' ') res += sym.ToString();
            res = res.ToLower();
            return res;
        }
        /// <summary>
        /// шифрование поля text
        /// с использованием таблицы кодировки символов
        /// </summary>
        /// <param name="code">таблица кодировки</param>
        /// <returns>зашифрованный текст</returns>
        public string Cipher(string[] code)
        {
            Array ar;
            string res = "";
            foreach (char sym in text)
            {
                int k = code[0].IndexOf(sym);
                if (k >= 0) res += code[1][k];
                else res += sym.ToString();
            }
            return res;
        }
        /// <summary>
        /// Проверка поля text,является ли он палиндромом
        ///  после преобразования Convert
        /// </summary>
        /// <returns> true, если палиндром</returns>
        public bool IsPalindrom()
        {
            string txt = Convert();
            for(int i=0, j = txt.Length-1; i<j; i++, j--)
                if(txt[i] != txt[j]) return false;
            return true;
        }
        /// <summary>
        /// Шифрование, заданное собственной таблицей кодировки 
        /// </summary>
        /// <returns>шифрованный текст</returns>
        public string Coding()
        {
            return Cipher(codeTable);
        }       
    }
}
