﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ConsoleInterface
{
    /// <summary>
    /// Наследник двух интерфейсов
    /// у методов которых Convert и Cipher
    /// сигнатуры совпадают.
    /// Методы Cipher склеиваются, 
    /// Convert - переименовываются
    /// </summary>
    class TwoInterfaces:IStrings,ITransform
    {
        //поля класса
        string text;
        static string[] codeTable =
        {
          "абвгдеёжзийклмнопрстуфхцчшщыьъэюя ,.!?;:",
          "ъьыщшчцхфуэюя ,.!?;:тсрпонмлкйабвгдеёжзи"
        };
        //Конструкторы
        public TwoInterfaces()
        {
            text = "Простой текст!";
        }
        public TwoInterfaces(string txt)
        {
            text = txt;
        }
        public string Text
        {
            get { return text; }
        }
        //Реализация интерфейсов
        /// <summary>
        /// Удаление пробелов поля text, 
        /// преобразование к нижнему регистру 
        /// </summary>
        /// <returns>преобразованная строка</returns>
        string IStrings.Convert()
        {            
            string res = "";
            foreach (char sym in text)            
                if (sym != ' ') res += sym.ToString();
            res = res.ToLower();
            return res;
        }
        
        /// <summary>
        /// Реверс строки, заданной полем text
        /// </summary>
        /// <returns>обращенная строка</returns>
        string ITransform.Convert()
        {            
            string res = "";
            for (int i = text.Length - 1; i >= 0; i--)
                res += text[i];
            return res;
        }
        //Переименование закрытых методов
        public string ConvertOne()
        {
            return ((IStrings)this).Convert();
        }
        public string ConvertTwo()
        {
            return ((ITransform)this).Convert();
        }
        //Склеивание метода Cipher двух интерфейсов
        public string Cipher(string[] code)
        {
            string s = text;
            string res = "";
            foreach (char sym in s)
            {
                int k = code[0].IndexOf(sym);
                if (k >= 0) res += code[1][k];
                else res += sym.ToString();
            }
            return res;
        }

        /// <summary>
        /// Проверка поля text,является ли он палиндромом
        ///  после преобразования Convert
        /// </summary>
        /// <returns> true, если палиндром</returns>
        public bool IsPalindrom()
        {
            string txt = ConvertOne();
            for(int i=0, j = txt.Length-1; i<j; i++, j--)
                if(txt[i] != txt[j]) return false;
            return true;
        }
        /// <summary>
        /// Шифрование, заданное таблицей кодировки 
        /// </summary>
        /// <returns>шифрованный текст</returns>
        public string Coding()
        {
            return Cipher(codeTable);
        }       
    }
   }

